/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { BinaryFileResource, BinaryFileService } from '@alfa-client/binary-file-shared';
import {
  HasLinkPipe,
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { FileUploadEditorComponent } from 'libs/ui/src/lib/ui/editor/file-upload-editor/file-upload-editor.component';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BinaryFileContainerComponent } from './../binary-file-container/binary-file-container.component';
import { BinaryFileAttachmentContainerComponent } from './binary-file-attachment-container.component';

describe('BinaryFileAttachmentContainerComponent', () => {
  let component: BinaryFileAttachmentContainerComponent;
  let fixture: ComponentFixture<BinaryFileAttachmentContainerComponent>;

  const binaryFileService = mock(BinaryFileService);
  const binaryFile: BinaryFileResource = createBinaryFileResource();
  const linkRelUploadAttachment: string = 'test';

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BinaryFileAttachmentContainerComponent,
        HasLinkPipe,
        MockComponent(BinaryFileContainerComponent),
        MockComponent(FileUploadEditorComponent),
      ],
      providers: [
        {
          provide: BinaryFileService,
          useValue: binaryFileService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(BinaryFileAttachmentContainerComponent);
    component = fixture.componentInstance;
    component.uploadStateResource = createStateResource(binaryFile);
    component.linkRelUploadAttachment = linkRelUploadAttachment;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('existing files', () => {
    it('should show existing files', () => {
      const binaryFile2: BinaryFileResource = createBinaryFileResource();

      component.existFiles = [binaryFile2, binaryFile];
      fixture.detectChanges();

      expect(component.fileList).toHaveLength(2);
      expect(component.fileList[0]).toBe(binaryFile2);
      expect(component.fileList[1]).toBe(binaryFile);
    });
  });

  describe('upload file', () => {
    const file: File = <any>{ name: 'TestDatei' };

    it('should call uploadAndGetFile', () => {
      component.uploadAndGetFile = jest.fn();

      component.uploadFile(file);

      expect(component.uploadAndGetFile).toHaveBeenCalledWith(file);
    });
  });

  describe('upload and get file', () => {
    const file: File = <any>{ name: 'TestDatei' };
    const responseStateResource: StateResource<BinaryFileResource> =
      createStateResource(binaryFile);

    beforeEach(() => {
      binaryFileService.uploadFile.mockReturnValue(of(responseStateResource));
    });

    it('should call service', () => {
      component.uploadFile(file);

      expect(binaryFileService.uploadFile).toHaveBeenCalledWith(
        binaryFile,
        linkRelUploadAttachment,
        file,
      );
    });

    it('should call doAfterFileUpload', (done) => {
      component.doAfterFileUpload = jest.fn();

      component.uploadAndGetFile(file).subscribe(() => {
        expect(component.doAfterFileUpload).toHaveBeenCalledWith(responseStateResource);
        done();
      });

      component.uploadFile(file);
    });
  });

  describe('doAfterFileUpload', () => {
    it('should add binaryFile', () => {
      component.doAfterFileUpload(createStateResource(binaryFile));

      expect(component.fileList).toHaveLength(1);
      expect(component.fileList[0]).toBe(binaryFile);
    });

    it('should NOT add binaryFile', () => {
      component.doAfterFileUpload(createEmptyStateResource());

      expect(component.fileList).toHaveLength(0);
    });
  });

  describe('delete attachment', () => {
    const existingFile: BinaryFileResource = createBinaryFileResource();

    beforeEach(() => {
      component.fileList = [existingFile];
    });

    it('should remove binaryFile', () => {
      component.deleteAttachment(existingFile);

      expect(component.fileList).toHaveLength(0);
    });

    it('should NOT remove binaryFile', () => {
      component.deleteAttachment(createBinaryFileResource());

      expect(component.fileList).toHaveLength(1);
      expect(component.fileList[0]).toBe(existingFile);
    });
  });
});
