import { getElementFromFixture } from '@alfa-client/test-utils';
import { ComponentFixture, fakeAsync, TestBed, tick } from '@angular/core/testing';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { PopupComponent } from './popup.component';

describe('PopupComponent', () => {
  let component: PopupComponent;
  let fixture: ComponentFixture<PopupComponent>;
  const popupButton: string = getDataTestIdOf('popup-button');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [PopupComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(PopupComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('togglePopup', () => {
    it('should change false to true', () => {
      component.isPopupOpen = false;

      component.togglePopup();

      expect(component.isPopupOpen).toBe(true);
    });

    it('should change true to false', () => {
      component.isPopupOpen = true;

      component.togglePopup();

      expect(component.isPopupOpen).toBe(false);
    });
  });

  describe('handleButtonClick', () => {
    beforeEach(() => {
      component.togglePopup = jest.fn();
      component.focusList = jest.fn();
    });

    it('should toggle popup', () => {
      component.handleButtonClick();

      expect(component.togglePopup).toHaveBeenCalled();
    });

    it('should focus list if popup is visible', () => {
      component.isPopupOpen = true;

      component.handleButtonClick();

      expect(component.focusList).toHaveBeenCalled();
    });

    it('should not focus list if popup is hidden', () => {
      component.handleButtonClick();

      expect(component.focusList).not.toHaveBeenCalled();
    });
  });

  describe('focusList', () => {
    it('should focus popup list item', fakeAsync(() => {
      component.isPopupOpen = true;
      fixture.detectChanges();
      component.popupListRef.nativeElement.focus = jest.fn();

      component.focusList();
      tick();

      expect(component.popupListRef.nativeElement.focus).toHaveBeenCalled();
    }));
  });

  describe('aria-expanded', () => {
    it('should be true if popup is open', () => {
      component.isPopupOpen = true;
      fixture.detectChanges();

      const buttonElement: HTMLElement = getElementFromFixture(fixture, popupButton);

      expect(buttonElement.getAttribute('aria-expanded')).toBe('true');
    });

    it('should be false if popup is closed', () => {
      component.isPopupOpen = false;
      fixture.detectChanges();

      const buttonElement: HTMLElement = getElementFromFixture(fixture, popupButton);

      expect(buttonElement.getAttribute('aria-expanded')).toBe('false');
    });
  });

  describe('isEscapeKey', () => {
    it('should return true', () => {
      const escapeKeyEvent: KeyboardEvent = {
        ...new KeyboardEvent('esc'),
        key: 'Escape',
      };

      const result: boolean = component.isEscapeKey(escapeKeyEvent);

      expect(result).toBe(true);
    });

    it('should return false', () => {
      const keyEvent: KeyboardEvent = new KeyboardEvent('whatever');

      const result: boolean = component.isEscapeKey(keyEvent);

      expect(result).toBe(false);
    });
  });

  describe('closePopupAndFocusButton', () => {
    beforeEach(() => {
      component.isPopupOpen = true;
      jest.spyOn(component.buttonRef.nativeElement, 'focus');
    });
    it('should close popup', () => {
      component.closePopupAndFocusButton();

      expect(component.isPopupOpen).toBe(false);
    });

    it('should focus button', () => {
      component.closePopupAndFocusButton();

      expect(component.buttonRef.nativeElement.focus).toHaveBeenCalled();
    });
  });

  describe('isPopupClosed', () => {
    it('should return true', () => {
      component.isPopupOpen = false;

      const result: boolean = component.isPopupClosed();

      expect(result).toBe(true);
    });

    it('should return false', () => {
      component.isPopupOpen = true;

      const result: boolean = component.isPopupClosed();

      expect(result).toBe(false);
    });
  });

  describe('onKeydownHandler', () => {
    const e: KeyboardEvent = new KeyboardEvent('test');

    beforeEach(() => {
      component.closePopupAndFocusButton = jest.fn();
      component.isEscapeKey = jest.fn();
    });

    describe('popup is closed', () => {
      beforeEach(() => {
        component.isPopupClosed = jest.fn().mockReturnValue(true);
      });

      it('should not check for escape key', () => {
        component.onKeydownHandler(e);

        expect(component.isEscapeKey).not.toHaveBeenCalled();
      });
    });

    describe('popup is open', () => {
      beforeEach(() => {
        component.isPopupClosed = jest.fn().mockReturnValue(false);
      });

      it('should check for escape key', () => {
        component.onKeydownHandler(e);

        expect(component.isEscapeKey).toHaveBeenCalled();
      });

      it('should handle escape key', () => {
        component.isEscapeKey = jest.fn().mockReturnValue(true);

        component.onKeydownHandler(e);

        expect(component.closePopupAndFocusButton).toHaveBeenCalled();
      });

      it('should not handle escape key', () => {
        component.onKeydownHandler(e);

        expect(component.closePopupAndFocusButton).not.toHaveBeenCalled();
      });
    });
  });
  describe('onClickHandler', () => {
    const e: MouseEvent = new MouseEvent('test');

    beforeEach(() => {
      component.closePopupAndFocusButton = jest.fn();
      component.buttonRef.nativeElement.contains = jest.fn();
    });

    describe('popup is closed', () => {
      beforeEach(() => {
        component.isPopupClosed = jest.fn().mockReturnValue(true);
      });

      it('should not check for button containing event target', () => {
        component.onClickHandler(e);

        expect(component.buttonRef.nativeElement.contains).not.toHaveBeenCalled();
      });
    });

    describe('popup is open', () => {
      beforeEach(() => {
        component.isPopupClosed = jest.fn().mockReturnValue(false);
      });

      it('should check for button containing event target', () => {
        component.onClickHandler(e);

        expect(component.buttonRef.nativeElement.contains).toHaveBeenCalled();
      });

      it('should handle click', () => {
        component.onClickHandler(e);

        expect(component.closePopupAndFocusButton).toHaveBeenCalled();
      });

      it('should not handle click', () => {
        component.buttonRef.nativeElement.contains = jest.fn().mockReturnValue(true);

        component.onClickHandler(e);

        expect(component.closePopupAndFocusButton).not.toHaveBeenCalled();
      });
    });
  });
});
