/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.attacheditem;

import java.util.Map;
import java.util.stream.Stream;

import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandBody;
import de.ozgcloud.alfa.common.command.CommandBodyMapper;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandService;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.kommentar.Kommentar;
import de.ozgcloud.alfa.loeschanforderung.LoeschAnforderung;
import de.ozgcloud.alfa.wiedervorlage.Wiedervorlage;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class VorgangAttachedItemService {

	private final CommandBodyMapper commandBodyMapper;
	private final CommandService commandService;
	private final ContextService contextService;

	private final VorgangAttachedItemRemoteService vorgangAttachedItemRemoteService;

	static final String WIEDERVORLAGE_ITEM_NAME = "Wiedervorlage";
	static final String KOMMENTAR_ITEM_NAME = "Kommentar";
	static final String LOESCH_ANFORDERUNG_ITEM_NAME = "LoeschAnforderung";

	public Command createNewWiedervorlage(Wiedervorlage wiedervorlage, String vorgangId) {
		return createNewVorgangAttachedItem(wiedervorlage, WIEDERVORLAGE_ITEM_NAME, vorgangId);
	}

	public Command createNewKommentar(Kommentar kommentar, String vorgangId) {
		return createNewVorgangAttachedItem(kommentar, KOMMENTAR_ITEM_NAME, vorgangId);
	}

	public Command createNewLoeschAnforderung(LoeschAnforderung loeschAnforderung, String vorgangId) {
		return createNewVorgangAttachedItem(loeschAnforderung, LOESCH_ANFORDERUNG_ITEM_NAME, vorgangId);
	}

	private Command createNewVorgangAttachedItem(CommandBody item, String itemName, String vorgangId) {
		var vorgangAttachedItem = buildVorgangAttachedItem(item, vorgangId, itemName);

		return commandService.createCommandDeprecated(buildCreateCommand(vorgangId, vorgangAttachedItem));
	}

	public boolean isLoeschAnforderung(String itemName) {
		return itemName.equals(LOESCH_ANFORDERUNG_ITEM_NAME);
	}

	CreateCommand buildCreateCommand(String vorgangId, CommandBody body) {
		return CreateCommand.builder()
				.vorgangId(vorgangId)
				.relationId(vorgangId)
				.order(CommandOrder.CREATE_ATTACHED_ITEM.name())
				.body(body)
				.build();
	}

	public Command editKommentar(Kommentar kommentar, String kommentarId, long kommentarVersion) {
		var vorgangAttachedItem = buildVorgangAttachedItem(kommentar, kommentar.getVorgangId(), KOMMENTAR_ITEM_NAME);

		return commandService.createCommand(
				buildUpdateCommand(kommentar.getVorgangId(), kommentarId, vorgangAttachedItem), kommentarVersion);
	}

	public Command editWiedervorlage(Wiedervorlage wiedervorlage, String wiedervorlageId, long version) {
		var vorgangAttachedItem = buildVorgangAttachedItem(wiedervorlage, wiedervorlage.getVorgangId(), WIEDERVORLAGE_ITEM_NAME);

		return commandService.createCommand(
				buildUpdateCommand(wiedervorlage.getVorgangId(), wiedervorlageId, vorgangAttachedItem), version);
	}

	VorgangAttachedItem buildVorgangAttachedItem(CommandBody body, String vorgangId, String itemName) {
		return VorgangAttachedItem.builder()
				.client(contextService.getClientName())
				.vorgangId(vorgangId)
				.itemName(itemName)
				.item(commandBodyMapper.fromObjectToMap(body))
				.build();
	}

	CreateCommand buildUpdateCommand(String vorgangId, String relationId, CommandBody body) {
		return CreateCommand.builder()
				.vorgangId(vorgangId)
				.relationId(relationId)
				.order(CommandOrder.UPDATE_ATTACHED_ITEM.name())
				.body(body)
				.build();
	}

	public Command setWiedervorlageDone(Wiedervorlage wiedervorlage, boolean done) {
		return commandService.createCommand(createPatchCommand(wiedervorlage, done), wiedervorlage.getVersion());
	}

	CreateCommand createPatchCommand(Wiedervorlage wiedervorlage, boolean done) {
		return getPatchCommandBuilder(wiedervorlage).body(createSetDoneBody(done)).relationVersion(wiedervorlage.getVersion()).build();
	}

	CreateCommand.CreateCommandBuilder getPatchCommandBuilder(Wiedervorlage wiedervorlage) {
		return CreateCommand.builder()
				.relationId(wiedervorlage.getId())
				.vorgangId(wiedervorlage.getVorgangId())
				.order(CommandOrder.PATCH_ATTACHED_ITEM.name());
	}

	private VorgangAttachedItem createSetDoneBody(boolean value) {
		return VorgangAttachedItem.builder().itemName(Wiedervorlage.class.getSimpleName()).item(Map.of(Wiedervorlage.DONE_FIELD, value)).build();
	}

	public Stream<VorgangAttachedItem> findLoeschAnforderung(String vorgangId) {
		return vorgangAttachedItemRemoteService.findVorgangAttachedItem(vorgangId, LOESCH_ANFORDERUNG_ITEM_NAME);
	}

	public VorgangAttachedItem getById(String id) {
		return vorgangAttachedItemRemoteService.getById(id);
	}

	public Command deleteAttachedItem(VorgangAttachedItem vorgangAttachedItem) {
		var createCommand = buildDeleteCommand(vorgangAttachedItem);

		return commandService.createCommand(createCommand, vorgangAttachedItem.getVersion());
	}

	CreateCommand buildDeleteCommand(VorgangAttachedItem vorgangAttachedItem) {
		return CreateCommand.builder()
				.vorgangId(vorgangAttachedItem.getVorgangId())
				.relationId(vorgangAttachedItem.getId())
				.relationVersion(vorgangAttachedItem.getVersion())
				.order(CommandOrder.DELETE_ATTACHED_ITEM.name())
				.body(vorgangAttachedItem)
				.build();
	}
}