/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.errorhandling;

import java.util.UUID;

import jakarta.validation.ConstraintViolationException;

import org.springframework.core.annotation.Order;
import org.springframework.http.HttpStatus;
import org.springframework.http.ProblemDetail;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseEntityExceptionHandler;

import de.ozgcloud.common.errorhandling.ExceptionUtil;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@RestControllerAdvice
@RequiredArgsConstructor
@Log4j2
@Order(98)
public class ExceptionController extends ResponseEntityExceptionHandler {

	static final String RUNTIME_MESSAGE_CODE = "generale.server_error";
	static final String RESOURCE_NOT_FOUNT_MESSAGE_CODE = "resource.not_found";
	static final String ACCESS_DENIED_MESSAGE_CODE = "generale.access_denied";

	private final ProblemDetailMapper problemDetailMapper;

	@ExceptionHandler(FunctionalException.class)
	@ResponseStatus(HttpStatus.BAD_REQUEST)
	public ApiError handleFunctionalException(FunctionalException e) {
		LOG.error("BadRequest", e);
		return ApiError.builder().issue(buildIssueForFunctionalException(e)).build();
	}

	private Issue buildIssueForFunctionalException(FunctionalException e) {
		return Issue.builder().message(e.getMessage()).messageCode(e.getErrorCode()).exceptionId(e.getExceptionId()).build();
	}

	@ExceptionHandler(AccessDeniedException.class)
	@ResponseStatus(HttpStatus.FORBIDDEN)
	public ApiError handleAccessDeniedException(AccessDeniedException e) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.error("Access Denied Exception: {}", messageWithExceptionId);

		return ApiError.builder().issue(buildIssueForAccessDeniedException(messageWithExceptionId, exceptionId)).build();
	}

	private Issue buildIssueForAccessDeniedException(String message, String exceptionId) {
		return Issue.builder().messageCode(ACCESS_DENIED_MESSAGE_CODE).message(message).exceptionId(exceptionId).build();
	}

	@ExceptionHandler(ResourceNotFoundException.class)
	@ResponseStatus(HttpStatus.NOT_FOUND)
	public ApiError handleResourceNotFoundException(ResourceNotFoundException e) {
		LOG.warn("Resource not found: {}", e.getMessage());
		return ApiError.builder().issue(buildIssueForResourceNotFoundException(e)).build();
	}

	private Issue buildIssueForResourceNotFoundException(ResourceNotFoundException e) {
		return Issue.builder().message(e.getMessage()).messageCode(RESOURCE_NOT_FOUNT_MESSAGE_CODE).exceptionId(e.getExceptionId()).build();
	}

	@ExceptionHandler(ConstraintViolationException.class)
	public ProblemDetail handleConstraintViolationException(ConstraintViolationException e) {
		LOG.warn("Validation Exception: {}", problemDetailMapper.buildMessageWithExceptionId(e));

		return problemDetailMapper.fromConstraintViolationException(e);
	}

	@ExceptionHandler(TechnicalException.class)
	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	public ApiError handleTechnicalException(TechnicalException e) {
		LOG.error("TechnicalException on Request", e);
		return buildRuntimeApiError(e.getMessage(), e.getExceptionId());
	}

	@ExceptionHandler(RuntimeException.class)
	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	public ApiError handleRuntimeException(RuntimeException e) {
		var exceptionId = createExceptionId();
		var messageWithExceptionId = ExceptionUtil.formatMessageWithExceptionId(e.getMessage(), exceptionId);
		LOG.error("RuntimeException on Request: " + messageWithExceptionId, e);
		return buildRuntimeApiError(messageWithExceptionId, exceptionId);
	}

	String createExceptionId() {
		return UUID.randomUUID().toString();
	}

	private ApiError buildRuntimeApiError(String message, String exceptionId) {
		return ApiError.builder().issue(createIssueForRuntimeException(message, exceptionId)).build();
	}

	private Issue createIssueForRuntimeException(String message, String exceptionId) {
		return Issue.builder().messageCode(RUNTIME_MESSAGE_CODE).message(message).exceptionId(exceptionId).build();
	}
}