/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.stream.Stream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.RepresentationModel;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.method.annotation.StreamingResponseBody;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileController;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.command.CommandController.CommandByRelationController;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@RestController
@RequestMapping(PostfachMailController.PATH)
public class PostfachMailController {

	public static final String PATH = "/api/postfachMails"; // NOSONAR
	public static final String PARAM_VORGANG_ID = "vorgangId";
	static final String PDF_NAME_TEMPLATE = "%s_%s_Nachrichten.pdf";
	static final DateTimeFormatter PDF_NAME_DATE_FORMATTER = DateTimeFormatter.ofPattern("yyyyMMdd");

	private final PostfachMailService postfachMailService;
	private final PostfachSettingsService postfachSettingsService;

	private final PostfachMailModelAssembler assembler;

	private final VorgangController vorgangController;
	private final BinaryFileController binaryFileController;

	@GetMapping(params = PARAM_VORGANG_ID)
	public RepresentationModel<EntityModel<PostfachSettings>> getAll(@RequestParam String vorgangId) {
		var vorgang = getVorgang(vorgangId);
		var postfachSetting = postfachSettingsService.getPostfachSettings(vorgang);
		return assembler.toCollectionModel(sort(postfachMailService.getAll(vorgangId)), vorgang, postfachSetting);
	}

	Stream<PostfachMail> sort(Stream<PostfachMail> nachrichten) {
		return nachrichten.sorted(new PostfachNachrichtComparator().reversed());
	}

	@GetMapping(params = PARAM_VORGANG_ID, produces = MediaType.APPLICATION_PDF_VALUE)
	public ResponseEntity<StreamingResponseBody> getAllAsPdf(@RequestParam String vorgangId) {
		var vorgang = getVorgang(vorgangId);

		return buildResponseEntity(vorgang, createDownloadStreamingBody(vorgang));
	}

	private VorgangWithEingang getVorgang(String vorgangId) {
		return vorgangController.getVorgang(vorgangId);
	}

	StreamingResponseBody createDownloadStreamingBody(VorgangWithEingang vorgang) {
		return out -> postfachMailService.getAllAsPdf(vorgang, out);
	}

	ResponseEntity<StreamingResponseBody> buildResponseEntity(VorgangWithEingang vorgang, StreamingResponseBody responseBody) {
		return ResponseEntity.ok()
				.header(HttpHeaders.CONTENT_DISPOSITION, buildPdfName(vorgang))
				.contentType(MediaType.APPLICATION_PDF)
				.body(responseBody);
	}

	private String buildPdfName(VorgangWithEingang vorgang) {
		return String.format(PDF_NAME_TEMPLATE, vorgang.getNummer(), LocalDate.now().format(PDF_NAME_DATE_FORMATTER));
	}

	public boolean isPostfachConfigured() {
		return postfachSettingsService.isPostfachConfigured();
	}

	@GetMapping("{nachrichtId}/attachments")
	public CollectionModel<EntityModel<OzgFile>> findAttachments(@PathVariable PostfachNachrichtId nachrichtId) {
		var postfachNachricht = postfachMailService.findById(nachrichtId);

		return binaryFileController.getFiles(postfachNachricht.getAttachments());
	}

	@RequiredArgsConstructor
	@RestController
	@RequestMapping(PostfachMailCommandController.POSTFACH_MAIL_COMMAND_PATH)
	public static class PostfachMailCommandController {
		static final String POSTFACH_MAIL_COMMAND_PATH = "/api/postfachMails/{postfachMailId}/commands"; // NOSONAR

		private final PostfachMailService service;
		private final PostfachNachrichtHelper helper;

		private final VorgangController vorgangController;

		private final CommandByRelationController commandByRelationController;

		@PostMapping
		public ResponseEntity<Void> createCommand(@PathVariable String postfachMailId, @RequestBody CreateCommand command) {
			var created = createCommand(service.findById(PostfachNachrichtId.from(postfachMailId)), command);

			if (created.getCommandOrder() == CommandOrder.RESEND_POSTFACH_MAIL) {
				service.resendPostfachMail(created.getId(), postfachMailId);
			}

			return ResponseEntity.created(linkTo(CommandController.class).slash(created.getId()).toUri()).build();
		}

		private Command createCommand(@NonNull PostfachMail mail, @NonNull CreateCommand command) {
			var completedCommand = command.toBuilder().vorgangId(mail.getVorgangId()).relationId(mail.getId()).build();

			if (completedCommand.getCommandOrder() == CommandOrder.SEND_POSTFACH_NACHRICHT) {
				var vorgang = getVorgang(mail.getVorgangId());
				completedCommand = addPostfachAddress(completedCommand, vorgang);
			}

			return commandByRelationController.createCommand(completedCommand);
		}

		private VorgangWithEingang getVorgang(String vorgangId) {
			return vorgangController.getVorgang(vorgangId);
		}

		private CreateCommand addPostfachAddress(CreateCommand command, VorgangWithEingang vorgang) {
			return command.toBuilder().body(helper.addPostfachAddressToBody((PostfachMail) command.getBody(), vorgang)).build();
		}
	}

	@RequiredArgsConstructor
	@RestController
	@RequestMapping(PostfachMailCommandByVorgangController.PATH)
	public static class PostfachMailCommandByVorgangController {

		static final String PATH = "/api/vorgangs/{vorgangId}/postfachMailCommands"; // NOSONAR

		private final PostfachMailService service;
		private final PostfachNachrichtHelper helper;
		private final VorgangController vorgangController;

		@PostMapping
		public ResponseEntity<Void> sendPostfachMail(@PathVariable String vorgangId, @RequestBody CreateCommand command) {
			command = command.toBuilder().vorgangId(vorgangId).relationId(vorgangId).build();

			var vorgang = getVorgang(vorgangId);
			command = addPostfachAddress(command, vorgang);

			var created = service.sendPostfachMail(command, vorgang.getVersion());

			return ResponseEntity.created(linkTo(CommandController.class).slash(created.getId()).toUri()).build();
		}

		private VorgangWithEingang getVorgang(String vorgangId) {
			return vorgangController.getVorgang(vorgangId);
		}

		private CreateCommand addPostfachAddress(CreateCommand command, VorgangWithEingang vorgang) {
			return command.toBuilder().body(helper.addPostfachAddressToBody((PostfachMail) command.getBody(), vorgang)).build();
		}
	}
}