package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.IanaLinkRelations;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.UriTemplate;

import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.command.CommandController.CommandByRelationController;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;

class CollaborationModelAssemblerTest {

	@Spy
	@InjectMocks
	private CollaborationModelAssembler assembler;

	@Mock
	private VorgangController vorgangController;

	@Nested
	class TestToModel {

		private static final String REL_ZUSTAENDIGE_STELLE = "zustaendigeStelle";
		private final Collaboration collaboration = CollaborationTestFactory.create();

		@Test
		void shouldHaveContent() {
			var entityModel = callAssembler();

			assertThat(entityModel.getContent()).isEqualTo(collaboration);
		}

		@Test
		void shouldHaveLinkToOrganisationsEinheit() {
			var entityModel = callAssembler();

			assertThat(entityModel.getLink(REL_ZUSTAENDIGE_STELLE)).get().extracting(Link::getHref)
					.isEqualTo(OrganisationsEinheitController.PATH + "/" + OrganisationsEinheitTestFactory.ID);
		}

		private EntityModel<Collaboration> callAssembler() {
			return assembler.toModel(collaboration);
		}
	}

	@Nested
	class TestToCollectionModel {
		private final Collaboration collaboration = CollaborationTestFactory.create();

		@Nested
		class OnNonEmptyCollaborationList {

			@Test
			void shouldCallToModel() {
				callAssembler();

				verify(assembler).toModel(collaboration);
			}

			@Test
			void shouldHaveEntityModel() {
				var entityModel = ModelBuilder.fromEntity(collaboration).buildModel();
				doReturn(entityModel).when(assembler).toModel(collaboration);

				var collectionModel = callAssembler();

				assertThat(collectionModel.getContent()).containsExactly(entityModel);
			}

			@Test
			void shouldHaveSelfLink() {
				var collectionModel = callAssembler();

				assertThat(collectionModel.getLink(IanaLinkRelations.SELF_VALUE)).get().extracting(Link::getHref)
						.isEqualTo(CollaborationController.PATH + "/" + VorgangHeaderTestFactory.ID + "/collaborations");
			}

			@Test
			void shouldNotHaveCreateCollaborationRequestLink() {
				var collectionModel = callAssembler();

				assertThat(collectionModel.getLink(CollaborationModelAssembler.REL_CREATE_COLLABORATION_REQUEST)).isEmpty();
			}

			private CollectionModel<EntityModel<Collaboration>> callAssembler() {
				return assembler.toCollectionModel(Stream.of(collaboration), VorgangHeaderTestFactory.ID);
			}
		}

		@Nested
		class OnEmptyCollaborationList {

			@Mock
			private Link createCollaborationRequestLink;

			@BeforeEach
			void mock() {
				doReturn(createCollaborationRequestLink).when(assembler).buildCreateCollaborationRequestLink(VorgangHeaderTestFactory.ID);
			}

			@Test
			void shouldHaveEmptyContent() {
				var collectionModel = callAssembler();

				assertThat(collectionModel.getContent()).isEmpty();
			}

			@Test
			void shouldHaveSelfLink() {
				var collectionModel = callAssembler();

				assertThat(collectionModel.getLink(IanaLinkRelations.SELF_VALUE)).get().extracting(Link::getHref)
						.isEqualTo(CollaborationController.PATH + "/" + VorgangHeaderTestFactory.ID + "/collaborations");
			}

			@Test
			void shouldHaveCreateCollaborationRequestLink() {
				var collectionModel = callAssembler();

				assertThat(collectionModel.getLinks()).contains(createCollaborationRequestLink);
			}

			private CollectionModel<EntityModel<Collaboration>> callAssembler() {
				return assembler.toCollectionModel(Stream.empty(), VorgangHeaderTestFactory.ID);
			}
		}
	}

	@Nested
	class TestBuildCreateCollaborationRequestLink {

		private final VorgangWithEingang vorgang = VorgangWithEingangTestFactory.create();

		@BeforeEach
		void mockVorgangController() {
			when(vorgangController.getVorgang(VorgangHeaderTestFactory.ID)).thenReturn(vorgang);
		}

		@Test
		void shouldCallVorgangController() {
			callAssembler();

			verify(vorgangController).getVorgang(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldHaveHrefToCreateCommand() {
			var expectedHref = UriTemplate.of(CommandByRelationController.COMMAND_BY_RELATION_PATH)
					.expand(VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.VERSION).toString();

			var link = callAssembler();

			assertThat(link).extracting(Link::getHref).isEqualTo(expectedHref);
		}

		@Test
		void shouldHaveCreateCollaborationRequestRelation() {
			var link = callAssembler();

			assertThat(link).extracting(Link::getRel).isEqualTo(CollaborationModelAssembler.REL_CREATE_COLLABORATION_REQUEST);
		}

		private Link callAssembler() {
			return assembler.buildCreateCollaborationRequestLink(VorgangHeaderTestFactory.ID);
		}
	}
}
