/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Map;

import jakarta.validation.ConstraintValidatorContext;

import org.hibernate.validator.constraintvalidation.HibernateConstraintValidatorContext;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.util.unit.DataSize;

class UploadBinaryFileSizeValidatorTest {

	@Spy
	@InjectMocks
	private UploadBinaryFileSizeValidator validator = new UploadBinaryFileSizeValidator();
	@Mock
	private BinaryFileProperties properties;

	@Nested
	class TestIsValid {

		@Mock
		private ConstraintValidatorContext context;
		@Mock
		private HibernateConstraintValidatorContext unwrappedContext;

		@BeforeEach
		void mockMaxValidSize() {
			doReturn(unwrappedContext).when(validator).getUnwrappedContext(any(), any());
			doReturn(DataSize.ofMegabytes(3)).when(validator).getMaxValidSize(anyString());
		}

		@Test
		void shouldCallGetMaxValidSize() {
			validator.isValid(UploadBinaryFileTestFactory.create(), context);

			verify(validator).getMaxValidSize(UploadBinaryFileTestFactory.FIELD);
		}

		@Test
		void shouldReturnTrue() {
			var result = validator.isValid(UploadBinaryFileTestFactory.create(), context);

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var result = validator.isValid(UploadBinaryFileTestFactory.createBuilder().size(DataSize.ofMegabytes(3).toBytes() + 1).build(), null);

			assertThat(result).isFalse();
		}

		@Nested
		class TestDynamicPayload {

			@Captor
			private ArgumentCaptor<DynamicViolationParameter> captor;

			@Test
			void shouldSetMaxValue() {
				validator.isValid(UploadBinaryFileTestFactory.create(), context);

				verify(unwrappedContext).withDynamicPayload(captor.capture());
				assertThat(String.valueOf(captor.getValue().getMap().get("max"))).isEqualTo("3");
			}
		}
	}

	@Nested
	class TestGetMaxValidSize {

		@Test
		void shouldReturnDefaultValueOnEmptyMap() {
			when(properties.getMaxFileSize()).thenReturn(Collections.emptyMap());

			var maxFileSize = validator.getMaxValidSize(BinaryFileTestFactory.FIELD);

			assertThat(maxFileSize.toBytes()).isEqualTo(DataSize.ofMegabytes(UploadBinaryFileSizeValidator.DEFAULT_MAX_SIZE).toBytes());
		}

		@Test
		void shouldReturnDefaultValueOnMissingField() {
			when(properties.getMaxFileSize()).thenReturn(Map.of("not match", DataSize.ofMegabytes(3)));

			var maxFileSize = validator.getMaxValidSize(BinaryFileTestFactory.FIELD);

			assertThat(maxFileSize.toBytes()).isEqualTo(DataSize.ofMegabytes(UploadBinaryFileSizeValidator.DEFAULT_MAX_SIZE).toBytes());
		}

		@Test
		void shouldReturnPropertiesValue() {
			when(properties.getMaxFileSize()).thenReturn(Map.of(BinaryFileTestFactory.FIELD, DataSize.ofMegabytes(3)));

			var maxFileSize = validator.getMaxValidSize(BinaryFileTestFactory.FIELD);

			assertThat(maxFileSize.toBytes()).isEqualTo(DataSize.ofMegabytes(3).toBytes());
		}
	}
}