package de.ozgcloud.alfa.common.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

class UserManagerUrlProviderTest {

	private static final String DUMMY_URL = "dummyUrl/";
	private static final String DUMMY_PROFILE_TEMPLATE = "dummyProfileTemplate";
	private static final String DUMMY_PROFILE_SEARCH_TEMPLATE = "dummyProfileSearchTemplate";

	@InjectMocks
	private UserManagerUrlProvider urlProvider;
	@Mock
	private UserManagerProperties userManagerProperties;

	@DisplayName("Get user profile template")
	@Nested
	class TestGetUserProfileTemplate {

		@Test
		void shouldGetUrlFromProperties() {
			urlProvider.getUserProfileTemplate();

			verify(userManagerProperties).getUrl();
		}

		@Test
		void shouldGetUserProfileTemplateFromProperties() {
			urlProvider.getUserProfileTemplate();

			verify(userManagerProperties).getProfileTemplate();
		}

		@Test
		void shouldReturnValue() {
			when(userManagerProperties.getUrl()).thenReturn(DUMMY_URL);
			when(userManagerProperties.getProfileTemplate()).thenReturn(DUMMY_PROFILE_TEMPLATE);

			var profileTemplate = urlProvider.getUserProfileTemplate();

			assertThat(profileTemplate).isEqualTo("dummyUrl/dummyProfileTemplate");
		}
	}

	@DisplayName("Get user profile search template")
	@Nested
	class TestGetUserProfileSearchTemplate {

		@Test
		void shouldGetUrlFromProperties() {
			urlProvider.getSearchUserProfilesTemplate();

			verify(userManagerProperties).getUrl();
		}

		@Test
		void shouldGetUserProfileSearchTemplateFromProperties() {
			urlProvider.getSearchUserProfilesTemplate();

			verify(userManagerProperties).getSearchTemplate();
		}

		@Test
		void shouldReturnValue() {
			when(userManagerProperties.getUrl()).thenReturn(DUMMY_URL);
			when(userManagerProperties.getSearchTemplate()).thenReturn(DUMMY_PROFILE_SEARCH_TEMPLATE);

			var profileTemplate = urlProvider.getSearchUserProfilesTemplate();

			assertThat(profileTemplate.toUriString()).isEqualTo("dummyUrl/dummyProfileSearchTemplate");
		}
	}

	@DisplayName("Is configured for user profile")
	@Nested
	class TestIsConfiguredForUserProfile {

		@Test
		void shouldGetUserManagerPropertiesUrl() {
			urlProvider.isConfiguredForUserProfile();

			verify(userManagerProperties).getUrl();
		}

		@Test
		void shouldGetUserManagerPropertiesProfileTemplate() {
			when(userManagerProperties.getUrl()).thenReturn(DUMMY_URL);

			urlProvider.isConfiguredForUserProfile();

			verify(userManagerProperties).getProfileTemplate();
		}

		@Test
		void shouldReturnTrueIfUrlAndProfileTemplateExists() {
			when(userManagerProperties.getUrl()).thenReturn(DUMMY_URL);
			when(userManagerProperties.getProfileTemplate()).thenReturn(DUMMY_PROFILE_TEMPLATE);

			var isConfigured = urlProvider.isConfiguredForUserProfile();

			assertThat(isConfigured).isTrue();
		}

		@Test
		void shouldReturnFalseIfUrlNotExists() {
			when(userManagerProperties.getUrl()).thenReturn(StringUtils.EMPTY);

			var isConfigured = urlProvider.isConfiguredForUserProfile();

			assertThat(isConfigured).isFalse();
		}

		@Test
		void shouldReturnFalseIfProfileTemplateNotExists() {
			when(userManagerProperties.getUrl()).thenReturn(DUMMY_URL);
			when(userManagerProperties.getProfileTemplate()).thenReturn(StringUtils.EMPTY);

			var isConfigured = urlProvider.isConfiguredForUserProfile();

			assertThat(isConfigured).isFalse();
		}
	}

	@DisplayName("Is configured for search user profile")
	@Nested
	class TestIsConfiguredForSearchUserProfile {

		@Test
		void shouldGetUserManagerPropertiesUrl() {
			urlProvider.isConfiguredForSearchUserProfile();

			verify(userManagerProperties).getUrl();
		}

		@Test
		void shouldGetUserManagerPropertiesSearchTemplate() {
			when(userManagerProperties.getUrl()).thenReturn(DUMMY_URL);

			urlProvider.isConfiguredForSearchUserProfile();

			verify(userManagerProperties).getSearchTemplate();
		}

		@Test
		void shouldReturnTrueIfUrlAndSearchTemplateExists() {
			when(userManagerProperties.getUrl()).thenReturn(DUMMY_URL);
			when(userManagerProperties.getSearchTemplate()).thenReturn(DUMMY_PROFILE_SEARCH_TEMPLATE);

			var isConfigured = urlProvider.isConfiguredForSearchUserProfile();

			assertThat(isConfigured).isTrue();
		}

		@Test
		void shouldReturnFalseIfUrlNotExists() {
			when(userManagerProperties.getUrl()).thenReturn(StringUtils.EMPTY);

			var isConfigured = urlProvider.isConfiguredForSearchUserProfile();

			assertThat(isConfigured).isFalse();
		}

		@Test
		void shouldReturnFalseIfProfileTemplateNotExists() {
			when(userManagerProperties.getUrl()).thenReturn(DUMMY_URL);
			when(userManagerProperties.getSearchTemplate()).thenReturn(StringUtils.EMPTY);

			var isConfigured = urlProvider.isConfiguredForSearchUserProfile();

			assertThat(isConfigured).isFalse();
		}
	}
}