/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.context.ApplicationContext;

import de.ozgcloud.alfa.ApplicationTestFactory;
import de.ozgcloud.alfa.vorgang.GrpcVorgangAttachedItemTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.vorgang.grpc.clientAttribute.ClientAttributeServiceGrpc.ClientAttributeServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcAccessPermission;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcDeleteClientAttributeRequest;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcSetClientAttributeRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceBlockingStub;

class WiedervorlageRemoteServiceTest {

	@Spy // NOSONAR
	@InjectMocks
	private WiedervorlageRemoteService service;

	@Mock
	private ClientAttributeServiceBlockingStub clientAttributeServiceStub;
	@Mock
	private VorgangAttachedItemServiceBlockingStub vorgangAttachedItemServiceStub;
	@Mock
	private WiedervorlageMapper mapper;
	@Mock
	private ApplicationContext applicationContext;

	@Nested
	class TestFindByVorgangId {

		private final GrpcFindVorgangAttachedItemResponse response = GrpcFindVorgangAttachedItemResponse.newBuilder()
				.addVorgangAttachedItems(GrpcVorgangAttachedItemTestFactory.create())
				.build();

		@Nested
		class ServiceMethod {

			@BeforeEach
			void initMocks() {
				when(vorgangAttachedItemServiceStub.find(any())).thenReturn(response);
			}

			@Test
			void shouldCallVorgangAttachedItemStub() {
				service.findByVorgangId(VorgangHeaderTestFactory.ID);

				verify(vorgangAttachedItemServiceStub).find(any(GrpcFindVorgangAttachedItemRequest.class));
			}

			@Nested
			class Mapper {

				@BeforeEach
				void mockGrpcMapper() {
					when(mapper.fromItem(any())).thenReturn(WiedervorlageTestFactory.create());
				}

				@Test
				void shouldCallForWiedervorlage() {
					callFindById();

					verify(mapper).fromItem(GrpcVorgangAttachedItemTestFactory.create());
				}

				private void callFindById() {
					var result = service.findByVorgangId(VorgangHeaderTestFactory.ID);
					collectStreamElementsToTriggerLazyStream(result);
				}

				private void collectStreamElementsToTriggerLazyStream(Stream<Wiedervorlage> stream) {
					stream.collect(Collectors.toList());
				}
			}
		}

		@Nested
		class BuildRequest {

			@Test
			void shouldSetVorgangId() {
				var request = buildRequest();

				assertThat(request.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
			}

			@Test
			void shouldSetItemName() {
				var request = buildRequest();

				assertThat(request.getItemName()).isEqualTo(WiedervorlageRemoteService.ITEM_NAME);
			}

			private GrpcFindVorgangAttachedItemRequest buildRequest() {
				return service.buildFindRequest(VorgangHeaderTestFactory.ID);
			}
		}
	}

	@Nested
	class TestGetById {

		private final GrpcVorgangAttachedItemRequest request = GrpcVorgangAttachedItemRequest.newBuilder()
				.setId(WiedervorlageTestFactory.ID).build();
		private final GrpcVorgangAttachedItemResponse response = GrpcVorgangAttachedItemResponse.newBuilder()
				.setVorgangAttachedItem(GrpcVorgangAttachedItemTestFactory.create())
				.build();

		@BeforeEach
		void initMocks() {
			when(vorgangAttachedItemServiceStub.getById(any())).thenReturn(response);
			when(mapper.fromItem(any())).thenReturn(WiedervorlageTestFactory.create());
		}

		@Test
		void shouldCallStub() {
			service.getById(WiedervorlageTestFactory.ID);

			verify(vorgangAttachedItemServiceStub).getById(request);
		}

		@Test
		void shouldCallWiedervorlageMapper() {
			service.getById(WiedervorlageTestFactory.ID);

			verify(mapper).fromItem(GrpcVorgangAttachedItemTestFactory.create());
		}

		@Test
		void shouldSetVorgangId() {
			var wiedervorlage = service.getById(WiedervorlageTestFactory.ID);

			assertThat(wiedervorlage.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}
	}

	@Nested
	class TestUpdateNextFrist {

		private static final LocalDate NEXT_FRIST = LocalDate.now();

		@Test
		void shouldSetOnExistingFrist() {
			service.updateNextFrist(VorgangHeaderTestFactory.ID, Optional.of(NEXT_FRIST));

			verify(service).setNextFrist(VorgangHeaderTestFactory.ID, NEXT_FRIST);
		}

		@Test
		void shouldDeleteOnNull() {
			doNothing().when(service).deleteNextFrist(any());

			service.updateNextFrist(VorgangHeaderTestFactory.ID, Optional.empty());

			verify(service).deleteNextFrist(VorgangHeaderTestFactory.ID);
		}

		@Nested
		class OnExisting {

			@Captor
			private ArgumentCaptor<GrpcSetClientAttributeRequest> requestCaptor;

			@Test
			void shouldCallSetServiceStub() {
				setNextFrist();

				verify(clientAttributeServiceStub).set(any(GrpcSetClientAttributeRequest.class));
			}

			@Test
			void shouldSetAccessPermissionToReadOnly() {
				setNextFrist();

				verify(clientAttributeServiceStub).set(requestCaptor.capture());
				assertThat(requestCaptor.getValue().getAttribute().getAccess()).isEqualTo(GrpcAccessPermission.READ_ONLY);
			}

			@Test
			void shouldSetAttributeName() {
				setNextFrist();

				verify(clientAttributeServiceStub).set(requestCaptor.capture());
				assertThat(requestCaptor.getValue().getAttribute().getAttributeName())
						.isEqualTo(WiedervorlageRemoteService.CLIENT_ATTRIBUTE_NEXT_WIEDERVORLAGE_FRIST);
			}

			@Test
			void shouldSetAttributeValue() {
				setNextFrist();

				verify(clientAttributeServiceStub).set(requestCaptor.capture());
				assertThat(requestCaptor.getValue().getAttribute().getValue().getStringValue())
						.isEqualTo(NEXT_FRIST.format(DateTimeFormatter.ISO_DATE));
			}

			private void setNextFrist() {
				service.setNextFrist(VorgangHeaderTestFactory.ID, NEXT_FRIST);
			}
		}

		@Nested
		class OnNonExisting {

			@Captor
			private ArgumentCaptor<GrpcDeleteClientAttributeRequest> requestCaptor;

			@BeforeEach
			void init() {
				when(applicationContext.getId()).thenReturn(ApplicationTestFactory.CONTEXT_ID);
			}

			@Test
			void shouldCallDeleteOnServiceStub() {
				deleteNextFrist();

				verify(clientAttributeServiceStub).delete(any(GrpcDeleteClientAttributeRequest.class));
			}

			@Test
			void shouldCallWithAttributeName() {
				deleteNextFrist();

				verify(clientAttributeServiceStub).delete(requestCaptor.capture());
				assertThat(requestCaptor.getValue().getAttributeName())
						.isEqualTo(WiedervorlageRemoteService.CLIENT_ATTRIBUTE_NEXT_WIEDERVORLAGE_FRIST);
			}

			@Test
			void shouldCallWithVorgangId() {
				deleteNextFrist();

				verify(clientAttributeServiceStub).delete(requestCaptor.capture());
				assertThat(requestCaptor.getValue().getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
			}

			@Test
			void shouldCallWithClient() {
				deleteNextFrist();

				verify(clientAttributeServiceStub).delete(requestCaptor.capture());
				assertThat(requestCaptor.getValue().getClientName()).isEqualTo(ApplicationTestFactory.CONTEXT_ID);
			}

			private void deleteNextFrist() {
				service.deleteNextFrist(VorgangHeaderTestFactory.ID);
			}
		}
	}
}