package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.AnlageDokumentTypeBuilder;
import de.ozgcloud.alfa.common.AnlageDokumentTypeTestFactory;
import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.common.HistorienProtokollInformationTypeTestFactory;
import de.ozgcloud.alfa.common.IdentifikationObjektTypeBuilder;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.export.AllgemeineMetadatenTypeTestFactory;
import de.ozgcloud.alfa.export.IdentifikationObjektTypeTestFactory;
import de.ozgcloud.alfa.vorgang.ZustaendigeStelleTestFactory;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.AnlageDokumentType;
import de.xoev.xdomea.HistorienProtokollInformationType;
import de.xoev.xdomea.IdentifikationObjektType;

class DokumentTypeBuilderTest {
	private final String fullName = UserProfileTestFactory.FULLNAME;
	private final Bescheid bescheid = BescheidTestFactory.create();
	private final OzgFile document = OzgFileTestFactory.createWithUniqueId();

	@Spy
	private final DokumentTypeBuilder builder = DokumentTypeBuilder.builder()
			.withBescheid(bescheid)
			.withOrganisationseinheitenId(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID)
			.withFullName(fullName)
			.withFiles(List.of(document));

	@Nested
	class TestBuild {
		private static final String TYP = "Bescheid";

		private MockedStatic<IdentifikationObjektTypeBuilder> identifikationObjektTypeBuilderMockedStatic;
		@Mock
		private IdentifikationObjektTypeBuilder identifikationObjektTypeBuilder;
		private final IdentifikationObjektType identifikationObjektType = IdentifikationObjektTypeTestFactory.create();

		private final HistorienProtokollInformationType historienProtokollInformation = HistorienProtokollInformationTypeTestFactory.create();
		private final AllgemeineMetadatenType metadatenType = AllgemeineMetadatenTypeTestFactory.create();
		private final AnlageDokumentType anlageDokumentDocument = AnlageDokumentTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			identifikationObjektTypeBuilderMockedStatic = mockStatic(IdentifikationObjektTypeBuilder.class);
			identifikationObjektTypeBuilderMockedStatic.when(IdentifikationObjektTypeBuilder::builder).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.withObjectID(BescheidTestFactory.ID)).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.build()).thenReturn(identifikationObjektType);
			doReturn(anlageDokumentDocument).when(builder).createAnlage(document);
		}

		@AfterEach
		void tearDown() {
			identifikationObjektTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetObjectId() {
			builder.build();

			verify(identifikationObjektTypeBuilder).withObjectID(bescheid.getId());
		}

		@Test
		void shouldBuildIdentifikationObjectType() {
			builder.build();

			verify(identifikationObjektTypeBuilder).build();
		}

		@Test
		void shouldHaveIdentifikation() {
			var dokument = builder.build();

			assertThat(dokument.getIdentifikation()).isEqualTo(identifikationObjektType);
		}

		@Test
		void shouldHaveTyp() {
			var dokument = builder.build();

			assertThat(dokument.getTyp()).isEqualTo(TYP);
		}

		@Test
		void shouldHaveBezugWithBetreff() {
			var subject = LoremIpsum.getInstance().getWords(5);
			builder.withBescheid(BescheidTestFactory.createBuilder().nachrichtSubject(subject).build());

			var dokument = builder.build();

			assertThat(dokument.getBezug()).isEqualTo(subject);
		}

		@Test
		void shouldHaveEmptyBezug() {
			builder.withBescheid(BescheidTestFactory.createBuilder().nachrichtSubject(null).build());

			var dokument = builder.build();

			assertThat(dokument.getBezug()).isEmpty();
		}

		@Test
		void shouldHaveMetadaten() {
			doReturn(metadatenType).when(builder).createAllgemeineMetadaten();

			var dokument = builder.build();

			assertThat(dokument.getAllgemeineMetadaten()).isEqualTo(metadatenType);
		}

		@Test
		void shouldHaveHistorienProtokollInformation() {
			doReturn(historienProtokollInformation).when(builder).createHistorienProtokollInformation();

			var dokument = builder.build();

			assertThat(dokument.getHistorienProtokollInformation()).containsExactly(historienProtokollInformation);
		}

		@Nested
		class TestWithoutAnlage {
			@Test
			void shouldHaveNoAnlage() {
				var dokument = builder.build();

				assertThat(dokument.getAnlage()).containsExactly(anlageDokumentDocument);
			}

		}

		@Nested
		class TestWithOneAnlage {
			private final AnlageDokumentType anlageDokumentAttachment = AnlageDokumentTypeTestFactory.create();
			private final OzgFile attachment = OzgFileTestFactory.createWithUniqueId();

			@Test
			void shouldHaveOneAnlage() {
				builder.withFiles(List.of(document, attachment));
				doReturn(anlageDokumentAttachment).when(builder).createAnlage(attachment);

				var dokument = builder.build();

				assertThat(dokument.getAnlage()).containsExactly(anlageDokumentDocument, anlageDokumentAttachment);
			}
		}
	}

	@Nested
	class TestCreateAllgemeineMetadaten {
		private final static String GENEHMIGT = "Bescheid wurde genehmigt.";
		private final static String ABGELEHNT = "Bescheid wurde abgelehnt.";

		@Test
		void shouldHaveBemerkungGenehmigt() {
			builder.withBescheid(BescheidTestFactory.createBuilder().bewilligt(true).build());

			var metadaten = builder.createAllgemeineMetadaten();

			assertThat(metadaten.getBemerkung()).isEqualTo(GENEHMIGT);
		}

		@Test
		void shouldHaveBemerkungAbgelehnt() {
			builder.withBescheid(BescheidTestFactory.createBuilder().bewilligt(false).build());

			var metadaten = builder.createAllgemeineMetadaten();

			assertThat(metadaten.getBemerkung()).isEqualTo(ABGELEHNT);
		}
	}

	@Nested
	class TestCreateHistorienProtokollInformation {
		private final static String GESPEICHERT = "Bescheid gespeichert";
		private final static String GESENDET = "Bescheid an Antragsteller gesendet";

		@Test
		void shouldHaveMetadatumName() {
			var text = LoremIpsum.getInstance().getHtmlParagraphs(1, 1);
			builder.withBescheid(BescheidTestFactory.createBuilder().nachrichtText(text).build());

			var protokollInfo = builder.createHistorienProtokollInformation();

			assertThat(protokollInfo.getMetadatumName()).isEqualTo(text);
		}

		@Test
		void shouldHaveEmptyMetadatumName() {
			builder.withBescheid(BescheidTestFactory.createBuilder().nachrichtText(null).build());

			var protokollInfo = builder.createHistorienProtokollInformation();

			assertThat(protokollInfo.getMetadatumName()).isEmpty();
		}

		@Test
		void shouldHaveAkteur() {
			var protokollInfo = builder.createHistorienProtokollInformation();

			assertThat(protokollInfo.getAkteur()).isEqualTo(fullName + "; " + ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);
		}

		@Test
		void shouldHaveDatumUhrzeit() {
			var protokollInfo = builder.createHistorienProtokollInformation();

			assertThat(protokollInfo.getDatumUhrzeit()).isEqualTo(DateConverter.toXmlGregorianCalendar(SentInfoTestFactory.SENT_AT));
		}

		@Test
		void shouldHaveGespeichertInAktion() {
			builder.withBescheid(BescheidTestFactory.createBuilder().sendBy(SendBy.MANUAL).build());

			var protokollInfo = builder.createHistorienProtokollInformation();

			assertThat(protokollInfo.getAktion()).isEqualTo(GESPEICHERT);
		}

		@Test
		void shouldHaveGesendetInAktion() {
			builder.withBescheid(BescheidTestFactory.createBuilder().sendBy(SendBy.NACHRICHT).build());

			var protokollInfo = builder.createHistorienProtokollInformation();

			assertThat(protokollInfo.getAktion()).isEqualTo(GESENDET);
		}
	}

	@Nested
	class TestCreateAnlage {
		private MockedStatic<AnlageDokumentTypeBuilder> anlageDokumentTypeBuilderMockedStatic;
		@Mock
		private AnlageDokumentTypeBuilder anlageDokumentTypeBuilder;
		private final AnlageDokumentType expectedAnlage = AnlageDokumentTypeTestFactory.create();
		private final OzgFile attachment = OzgFileTestFactory.createWithUniqueId();

		@BeforeEach
		void setUp() {
			anlageDokumentTypeBuilderMockedStatic = mockStatic(AnlageDokumentTypeBuilder.class);
			anlageDokumentTypeBuilderMockedStatic.when(AnlageDokumentTypeBuilder::builder).thenReturn(anlageDokumentTypeBuilder);
			when(anlageDokumentTypeBuilder.withOzgFile(attachment)).thenReturn(anlageDokumentTypeBuilder);
			when(anlageDokumentTypeBuilder.build()).thenReturn(expectedAnlage);
		}

		@AfterEach
		void tearDown() {
			anlageDokumentTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetOzgFile() {
			callCreateAnlage();

			verify(anlageDokumentTypeBuilder).withOzgFile(attachment);
		}

		@Test
		void shouldBuildAnlageDokumentType() {
			callCreateAnlage();

			verify(anlageDokumentTypeBuilder).build();
		}

		@Test
		void shouldReturnBuiltAnlage() {
			var resultAnlage = callCreateAnlage();

			assertThat(resultAnlage).isEqualTo(expectedAnlage);
		}

		private AnlageDokumentType callCreateAnlage() {
			return builder.createAnlage(attachment);
		}
	}
}
