package de.ozgcloud.alfa.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.MockedStatic;

import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;

class PrimaerdokumentTypeBuilderTest {

	private final PrimaerdokumentTypeBuilder builder = PrimaerdokumentTypeBuilder.builder().withOzgFile(OzgFileTestFactory.createWithUniqueId());

	@Nested
	class TestBuild {

		private MockedStatic<ExportFilenameGenerator> exportFilenameGeneratorMockedStatic;

		private static final String ERSTELLER = "Ersteller";
		private static final String GENERATED_FILE_NAME = "file_name";
		private static final ZonedDateTime CREATED_AT = ZonedDateTime.now();
		private final OzgFile ozgFile = OzgFileTestFactory.createWithUniqueId();

		@BeforeEach
		void setUp() {
			exportFilenameGeneratorMockedStatic = mockStatic(ExportFilenameGenerator.class);
			exportFilenameGeneratorMockedStatic.when(() -> ExportFilenameGenerator.generateExportFilename(ozgFile)).thenReturn(GENERATED_FILE_NAME);
		}

		@AfterEach
		void tearDown() {
			exportFilenameGeneratorMockedStatic.close();
		}

		@Test
		void shouldGenerateExportFilename() {
			builder.withOzgFile(ozgFile).build();

			exportFilenameGeneratorMockedStatic.verify(() -> ExportFilenameGenerator.generateExportFilename(ozgFile));
		}

		@Test
		void shouldHaveDateiname() {
			var primaerdokumentType = builder.withOzgFile(ozgFile).build();

			assertThat(primaerdokumentType.getDateiname()).isEqualTo(GENERATED_FILE_NAME);
		}

		@Test
		void shouldNotHaveDateiname() {
			var primaerdokumentType = builder.build();

			assertThat(primaerdokumentType.getDateiname()).isNull();
		}

		@Test
		void shouldHaveDateinameOriginal() {
			var primaerdokumentType = builder.withOzgFile(ozgFile).build();

			assertThat(primaerdokumentType.getDateinameOriginal()).isEqualTo(OzgFileTestFactory.NAME);
		}

		@Test
		void shouldHaveErsteller() {
			var primaerdokumentType = builder.withErsteller(ERSTELLER).build();

			assertThat(primaerdokumentType.getErsteller()).isEqualTo(ERSTELLER);
		}

		@Test
		void shouldNotHaveErsteller() {
			var primaerdokumentType = builder.build();

			assertThat(primaerdokumentType.getErsteller()).isNull();
		}

		@Test
		void shouldHaveDatumUhrzeit() {
			var primaerdokumentType = builder.withCreatedAt(CREATED_AT).build();

			assertThat(primaerdokumentType.getDatumUhrzeit()).isEqualTo(DateConverter.toXmlGregorianCalendar(CREATED_AT));
		}

		@Test
		void shouldNotHaveDatumUhrzeit() {
			var primaerdokumentType = builder.build();

			assertThat(primaerdokumentType.getDatumUhrzeit()).isNull();
		}

		@Test
		void shouldThrowIllegalgStateException() {
			builder.withOzgFile(null);

			assertThatThrownBy(builder::build).isInstanceOf(IllegalStateException.class);
		}
	}

}