package de.ozgcloud.alfa.file;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Random;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.IdentifikationObjektTypeBuilder;
import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.common.VersionTypeBuilder;
import de.ozgcloud.alfa.common.VersionTypeTestFactory;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.export.AllgemeineMetadatenTypeTestFactory;
import de.ozgcloud.alfa.export.IdentifikationObjektTypeTestFactory;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.VersionType;

class DokumentTypeBuilderTest {

	private static final Long DOKUMENT_ORDINAL_NUMBER = new Random().nextLong();
	private static final String FORM_ENGINE_NAME = "DUMMY_NAME";
	private final OzgFile ozgFile = OzgFileTestFactory.create();

	@Spy
	private final DokumentTypeBuilder builder = DokumentTypeBuilder.builder()
			.withOzgFile(ozgFile)
			.withFormEngineName(FORM_ENGINE_NAME)
			.withOrdinalNumber(DOKUMENT_ORDINAL_NUMBER);

	private MockedStatic<UUIDConverter> uuidConverter;

	@BeforeEach
	void init() {
		uuidConverter = mockStatic(UUIDConverter.class);
	}

	@AfterEach
	void cleanup() {
		uuidConverter.close();
	}

	@Nested
	class TestBuild {

		private final IdentifikationObjektType identifikationObjekt = IdentifikationObjektTypeTestFactory.create();
		private final AllgemeineMetadatenType allgemeineMetadaten = AllgemeineMetadatenTypeTestFactory.create();
		private final VersionType versionType = VersionTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			doReturn(identifikationObjekt).when(builder).createIdentifikation();
			doReturn(versionType).when(builder).createVersionType();
			doReturn(allgemeineMetadaten).when(builder).createAllgemeineMetadaten();
		}

		@Test
		void shouldCreateIdentifikation() {
			builder.build();

			verify(builder).createIdentifikation();
		}

		@Test
		void shouldHaveIdentifikation() {
			var dokument = builder.build();

			assertThat(dokument.getIdentifikation()).isEqualTo(identifikationObjekt);
		}

		@Test
		void shouldCreateAllgemeineMetadaten() {
			builder.build();

			verify(builder).createAllgemeineMetadaten();
		}

		@Test
		void shouldHaveAllgemeineMetadaten() {
			var dokument = builder.build();

			assertThat(dokument.getAllgemeineMetadaten()).isEqualTo(allgemeineMetadaten);
		}

		@Test
		void shouldCreateVersion() {
			builder.build();

			verify(builder).createVersionType();
		}

		@Test
		void shouldHaveVersion() {
			var dokumentType = builder.build();

			assertThat(dokumentType.getVersion()).containsExactly(versionType);
		}

	}

	@Nested
	class TestCreateIdentifikation {

		private MockedStatic<IdentifikationObjektTypeBuilder> identifikationObjektTypeBuilderMockedStatic;
		@Mock
		private IdentifikationObjektTypeBuilder identifikationObjektTypeBuilder;
		private final IdentifikationObjektType identifikationObjektType = IdentifikationObjektTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			identifikationObjektTypeBuilderMockedStatic = mockStatic(IdentifikationObjektTypeBuilder.class);
			identifikationObjektTypeBuilderMockedStatic.when(IdentifikationObjektTypeBuilder::builder).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.withObjectID(OzgFileTestFactory.ID.toString())).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.withOrdinalNumber(DOKUMENT_ORDINAL_NUMBER))
					.thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.build()).thenReturn(identifikationObjektType);
		}

		@AfterEach
		void tearDown() {
			identifikationObjektTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetObjectId() {
			builder.createIdentifikation();

			verify(identifikationObjektTypeBuilder).withObjectID(ozgFile.getId().toString());
		}

		@Test
		void shouldSetNummerImUebergeordnetenContainer() {
			builder.createIdentifikation();

			verify(identifikationObjektTypeBuilder).withOrdinalNumber(DOKUMENT_ORDINAL_NUMBER);
		}

		@Test
		void shouldBuildIdentifikationObjectType() {
			builder.createIdentifikation();

			verify(identifikationObjektTypeBuilder).build();
		}

		@Test
		void shouldReturnBuiltIdentifikation() {
			var resultIdentifikation = builder.createIdentifikation();

			assertThat(resultIdentifikation).isEqualTo(identifikationObjektType);
		}
	}

	@Nested
	class TestCreateAllgemeineMetadaten {

		@Test
		void shouldHaveBetreff() {
			var allgemeineMetadaten = builder.createAllgemeineMetadaten();

			assertThat(allgemeineMetadaten.getBetreff()).isEqualTo(OzgFileTestFactory.NAME);
		}

		@Test
		void shouldHaveEmptyKennzeichen() {
			var allgemeineMetadaten = builder.createAllgemeineMetadaten();

			assertThat(allgemeineMetadaten.getKennzeichen()).isEmpty();
		}

		@Test
		void shouldHaveEmptyBemerkung() {
			var allgemeineMetadaten = builder.createAllgemeineMetadaten();

			assertThat(allgemeineMetadaten.getBemerkung()).isEmpty();
		}

		@Test
		void shouldHaveMedium() {
			var allgemeineMetadaten = builder.createAllgemeineMetadaten();

			assertThat(allgemeineMetadaten.getMedium().getCode()).isEqualTo(DokumentTypeBuilder.ALLGEMEINE_METADATEN_MEDIUM_CODE);
		}

	}

	@Nested
	class CreateVersionType {

		private MockedStatic<VersionTypeBuilder> versionTypeBuilderMockedStatic;
		@Mock
		private VersionTypeBuilder versionTypeBuilder;
		private final VersionType version = VersionTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			versionTypeBuilderMockedStatic = mockStatic(VersionTypeBuilder.class);
			versionTypeBuilderMockedStatic.when(VersionTypeBuilder::builder).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.withErsteller(FORM_ENGINE_NAME)).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.withOzgFile(ozgFile)).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.withSonstigerName(StringUtils.EMPTY)).thenReturn(versionTypeBuilder);
			when(versionTypeBuilder.build()).thenReturn(version);
		}

		@AfterEach
		void tearDown() {
			versionTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetOzgFile() {
			callCreateVersionType();

			verify(versionTypeBuilder).withOzgFile(ozgFile);
		}

		@Test
		void shouldSetErsteller() {
			callCreateVersionType();

			verify(versionTypeBuilder).withErsteller(FORM_ENGINE_NAME);
		}

		@Test
		void shouldSetSonstigerName() {
			callCreateVersionType();

			verify(versionTypeBuilder).withSonstigerName(StringUtils.EMPTY);
		}

		@Test
		void shouldBuildVersionType() {
			callCreateVersionType();

			verify(versionTypeBuilder).build();
		}

		@Test
		void shouldReturnBuiltVersion() {
			var resultVersionType = callCreateVersionType();

			assertThat(resultVersionType).isEqualTo(version);
		}

		private VersionType callCreateVersionType() {
			return builder.createVersionType();
		}
	}

}