package de.ozgcloud.alfa.kommentar;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;

import javax.xml.datatype.XMLGregorianCalendar;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.alfa.common.AnlageDokumentTypeBuilder;
import de.ozgcloud.alfa.common.AnlageDokumentTypeTestFactory;
import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.common.HistorienProtokollInformationTypeTestFactory;
import de.ozgcloud.alfa.common.IdentifikationObjektTypeBuilder;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.export.IdentifikationObjektTypeTestFactory;
import de.ozgcloud.alfa.vorgang.ZustaendigeStelleTestFactory;
import de.xoev.xdomea.AnlageDokumentType;
import de.xoev.xdomea.DokumentType;
import de.xoev.xdomea.HistorienProtokollInformationType;
import de.xoev.xdomea.IdentifikationObjektType;

class DokumentTypeBuilderTest {

	@Spy
	private DokumentTypeBuilder builder = DokumentTypeBuilder.builder().withKommentar(KommentarTestFactory.create());

	@Nested
	class TestBuild {

		private MockedStatic<IdentifikationObjektTypeBuilder> identifikationObjektTypeBuilderMockedStatic;
		@Mock
		private IdentifikationObjektTypeBuilder identifikationObjektTypeBuilder;
		private final IdentifikationObjektType identifikationObjektType = IdentifikationObjektTypeTestFactory.create();

		private final OzgFile ozgFile = OzgFileTestFactory.create();
		private final AnlageDokumentType anlageDokumentType = AnlageDokumentTypeTestFactory.create();

		private final HistorienProtokollInformationType historienProtokollInformationType = HistorienProtokollInformationTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			identifikationObjektTypeBuilderMockedStatic = mockStatic(IdentifikationObjektTypeBuilder.class);
			identifikationObjektTypeBuilderMockedStatic.when(IdentifikationObjektTypeBuilder::builder).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.withObjectID(KommentarTestFactory.ID)).thenReturn(identifikationObjektTypeBuilder);
			when(identifikationObjektTypeBuilder.build()).thenReturn(identifikationObjektType);
			doReturn(historienProtokollInformationType).when(builder).createHistorie();
		}

		@AfterEach
		void tearDown() {
			identifikationObjektTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetObjectId() {
			builder.build();

			verify(identifikationObjektTypeBuilder).withObjectID(KommentarTestFactory.ID);
		}

		@Test
		void shouldBuildIdentifikationObjectType() {
			builder.build();

			verify(identifikationObjektTypeBuilder).build();
		}

		@Nested
		class WithoutKommentareAttachments {

			@Test
			void shouldHaveIdentifikation() {
				var dokumentType = builder.build();

				assertThat(dokumentType.getIdentifikation()).isEqualTo(identifikationObjektType);
			}

			@Test
			void shouldHaveTypNotiz() {
				var dokumentType = builder.build();

				assertThat(dokumentType.getTyp()).isEqualTo(DokumentTypeBuilder.TYP);
			}
		}

		@Nested
		class WithKommentareAttachments {

			@BeforeEach
			void setUp() {
				doReturn(anlageDokumentType).when(builder).createAnlage(ozgFile);
			}

			@Test
			void shouldCreateAnlagenForKommentarAttachments() {
				builder.withKommentarAttachments(List.of(ozgFile)).build();

				verify(builder).createAnlage(ozgFile);
			}

			@Test
			void shouldHaveAnlagen() {
				var dokumentType = builder.withKommentarAttachments(List.of(ozgFile)).build();

				assertThat(dokumentType.getAnlage()).contains(anlageDokumentType);
			}
		}

		@Test
		void shouldHaveHistorienProtokoll() {
			DokumentType dokumentType = builder.build();

			assertThat(dokumentType.getHistorienProtokollInformation().get(0)).isEqualTo(historienProtokollInformationType);
		}

		@Test
		void shouldHaveOneHistorienProtokoll() {
			DokumentType dokumentType = builder.build();

			assertThat(dokumentType.getHistorienProtokollInformation()).size().isEqualTo(1);
		}

		@Test
		void shouldCreateHistorienProtokoll() {
			builder.build();

			verify(builder).createHistorie();
		}

	}

	@Nested
	class TestCreateAnlage {

		private final OzgFile ozgFile = OzgFileTestFactory.create();

		private MockedStatic<AnlageDokumentTypeBuilder> anlageDokumentTypeBuilderMockedStatic;
		@Mock
		private AnlageDokumentTypeBuilder anlageDokumentTypeBuilder;
		private AnlageDokumentType expectedAnlage = AnlageDokumentTypeTestFactory.create();

		@BeforeEach
		void setUp() {
			anlageDokumentTypeBuilderMockedStatic = mockStatic(AnlageDokumentTypeBuilder.class);
			anlageDokumentTypeBuilderMockedStatic.when(AnlageDokumentTypeBuilder::builder).thenReturn(anlageDokumentTypeBuilder);
			when(anlageDokumentTypeBuilder.withOzgFile(ozgFile)).thenReturn(anlageDokumentTypeBuilder);
			when(anlageDokumentTypeBuilder.build()).thenReturn(expectedAnlage);
		}

		@AfterEach
		void tearDown() {
			anlageDokumentTypeBuilderMockedStatic.close();
		}

		@Test
		void shouldSetOzgFile() {
			callCreateAnlage();

			verify(anlageDokumentTypeBuilder).withOzgFile(ozgFile);
		}

		@Test
		void shouldBuildAnlageDokumentType() {
			callCreateAnlage();

			verify(anlageDokumentTypeBuilder).build();
		}

		@Test
		void shouldReturnBuiltAnlage() {
			var resultAnlage = callCreateAnlage();

			assertThat(resultAnlage).isEqualTo(expectedAnlage);
		}

		private AnlageDokumentType callCreateAnlage() {
			return builder.createAnlage(ozgFile);
		}
	}

	@Nested
	class TestWithOrganisationseinheitenID {
		@Test
		void shouldReturnBuilder() {
			var result = builder.withOrganisationseinheitenID(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

			assertThat(result).isNotNull();
		}
	}

	@Nested
	class TestWithAuthorFullName {
		@Test
		void shouldReturnBuilder() {
			var result = builder.withAuthorFullName(UserProfileTestFactory.FULLNAME);

			assertThat(result).isNotNull();
		}
	}

	@Nested
	class TestCreateHistorie {

		Kommentar kommentar = KommentarTestFactory.create();

		@BeforeEach
		void setupBuilder() {
			builder.withKommentar(kommentar);
		}

		@Test
		void shouldHaveMetadatumName() {
			var historie = builder.createHistorie();

			assertThat(historie.getMetadatumName()).isEqualTo(KommentarTestFactory.TEXT);
		}

		@Test
		void shouldHaveAkteur() {
			String expectedAkteurName = LoremIpsum.getInstance().getWords(5);
			doReturn(expectedAkteurName).when(builder).createAkteur();

			var historie = builder.createHistorie();

			assertThat(historie.getAkteur()).isEqualTo(expectedAkteurName);
		}

		@Test
		void shouldHaveDatumUhrzeit() {
			var historie = builder.createHistorie();

			assertThat(historie.getDatumUhrzeit()).isEqualTo(createExpectedDatumUhrzeit());
		}

		@Test
		void shouldHaveAktion() {
			var historie = builder.createHistorie();

			assertThat(historie).hasFieldOrPropertyWithValue("aktion", DokumentTypeBuilder.AKTION);
		}

		private XMLGregorianCalendar createExpectedDatumUhrzeit() {
			return DateConverter.toXmlGregorianCalendar(kommentar.getCreatedAt());
		}
	}

	@Nested
	class TestCreateAkteur {
		@Test
		void shouldCreateName() {
			builder.withAuthorFullName(UserProfileTestFactory.FULLNAME);
			builder.withOrganisationseinheitenID(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID);

			String akteur = builder.createAkteur();

			assertThat(akteur).isEqualTo(createExpectedAkteurName());
		}

		private String createExpectedAkteurName() {
			return UserProfileTestFactory.FULLNAME + "; " + ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID;
		}

	}

}